import React from 'react';
import PropTypes from 'prop-types';

import { storiesOf } from '@storybook/react';
import { withInfo } from '@storybook/addon-info';

import pure from '../../src/HOC/pure';

const getRandFragment = () => Math.round(Math.random() * 255);
const getRandColor = () => `rgb(${getRandFragment()}, ${getRandFragment()}, ${getRandFragment()})`;

const UnpureComponent = () => (
    <div style={{ color: getRandColor() }}>I rerendered at {Date.now()}</div>
);

class StatfulWrapper extends React.Component {
    static propTypes = {
        children: PropTypes.node.isRequired,
    }

    state = {
        data: {
            foo: 'bar',
            baz: 696,
            ras: {
                al: 'ghul',
            },
        },
    };

    confirmNewState() {
        try {
            const data = JSON.parse(this.input.value);
            this.setState({ data });
        } catch (e) {
            alert(`State is invalid, fix it: ${e.message}`);
        }
    }

    render() {
        const { data } = this.state;
        return (
            <div>
                <h3>State value</h3>
                <textarea
                    defaultValue={JSON.stringify(data, null, 4)}
                    ref={input => {
                        this.input = input;
                    }}
                    cols={100}
                    rows={10}
                    style={{ fontSize: '1.2em' }}
                />
                <br />
                <button onClick={() => this.confirmNewState()}>Set new state</button>

                <p>
                    {React.cloneElement(React.Children.only(this.props.children), { data })}
                </p>
            </div>
        );
    }
}

storiesOf('Pure', module)
    .add('pure', withInfo()(() => {
        const PureComponent = pure(UnpureComponent);
        return (
            <StatfulWrapper>
                <PureComponent />
            </StatfulWrapper>
        );
    }))
    .add('unpure', withInfo()(() => {
        return (
            <StatfulWrapper>
                <UnpureComponent />
            </StatfulWrapper>
        );
    }));
